
from lib.exceptions import *

'''
Exception numbers plan:

0    -  99: Xeek internal errors: network, protocol, ...
100  - 199: User input errors: invalid parameters
'''

class XeekAPIException(XeekException):
  def __init__(self, error_code, error_msg):
    XeekException.__init__(self, error_msg)
    self._error_code = error_code
    
  @staticmethod
  def checkOrRaise(res):
    """Raise an exception if the specified result is an error"""
    if(type(res) == type(dict()) and 'error_num' in res):
    
      error_num = int(res['error_num'])
    
      # Search the appropriate class in XeekAPIServerExceptions
      if(error_num in XeekAPIServerExceptions):
        raise XeekAPIServerExceptions[error_num](error_num, res['error_msg'])
      
      # Didn't found it - raise a standard exception
      else:
        raise XeekAPIException(error_num, res['error_msg'])
  
  def __str__(self):
    return "Error: "+self._error_msg
    
class LostConnection(XeekAPIException):
  def __init__(self):
    XeekAPIException.__init__(self, 0, "Can't connect to server.")

class ParsingError(XeekAPIException):
  def __init__(self, response):
    XeekAPIException.__init__(self, 1, "Unable to parse response. Is this a valid Xeek API server?")

class InvalidParameter(XeekAPIException):
  pass
  

# Xeek API Server Exceptions
XeekAPIServerExceptions = { 100: InvalidParameter }
