#!/usr/bin/python
# -*- coding: iso-8859-15 -*-

import re

from exceptions import *
from state import *
from token import *
  
class Parser:

  def __init__(self):
    """Constructor"""
    
    self.commands = []
    
    # Readline support
    try:
      import readline
      readline.set_completer(self.completer)
      readline.parse_and_bind('tab: menu-complete')
      readline.set_completer_delims("\t\n`~!@#$%^&*()-=+[{]}\\|;:\'\",<>/?")
    except Exception:
      print "Warning: No readline support for your platform!"
    
    
  def addCommand(self, token):
    """Add a command to the parser"""
    self.commands.append(token)
    
  
  def parseAndExecute(self, text):
    """
    Parse a command and execute the appropriate function.
    @param text typed command
    """
    
    tab_cmd = re.compile('\s+').split(text.rstrip())
    n = len(tab_cmd)
    
    # Handle empty command
    if(tab_cmd == ['']):
      return
    
    i = 0
    tab = dict([(token.text, token) for token in self.commands])
    
    end_command = None
    
    # For each token of the typed command
    while(i < n):
      word = tab_cmd[i]
      
      # If this token is in tab, it is authorized
      if(tab.has_key(word)):
      
        # If this token is final and can be followed by anything else, return now
        if tab[word].isUnilimited():
          return tab[word].endCommand(tab_cmd)
      
        # It is the last token ?
        if(i == n-1):
          
          # If the corresponding token is not final, error
          if(not tab[word].isFinal()):
            raise InvalidCommand()
          
          else: # Call the end callback
            return tab[word].endCommand(tab_cmd)
            
        # Other tokens are coming (but not from a token list) => continue
        else:
          tab = tab[word].getNextTokens()
          
      else:
        raise InvalidCommand()
      
      i += 1
    
    
  def completer(self, text, state):
    """Auto-completion function, called when the used presses <tab>"""
    # Handle multiple words
    tab_cmd = re.compile('\s+').split(text)
    n = len(tab_cmd)
    
    try:
      i = 0
      tab = dict([(token.text, token) for token in self.commands])
      
      # For each token before the last one
      while(i < n - 1):
        word = tab_cmd[i]
        
        # Is this token authorized?
        if(tab.has_key(word)):
          tab = tab[word].getNextTokens()
        
        else:
          return None
        
        i += 1
        
      
      # Try to autocomplete the last word
      if(n >= 2):
        matches = self.find_matches(tab_cmd[n-1], tab.keys())
        matches = [text[:len(text)-len(tab_cmd[n-1])] + m for m in matches]
        
      # Only one word
      else:
        matches = self.find_matches(text, tab.keys())
    
      return matches[state]
    except IndexError:
      return None
     
    
  def listCommands(self):
    """Return the list of all registered commands"""
    
    res = []
    for t in self.commands:
      res += t.getPossibleCommands()
    res.sort()
    return res
    
    
  def getHelp(self, tab_cmd):
    """
    Get help about a command
    @param text typed command
    @return {doc, usage}
    """
    
    def _getHelp(method):
      res = {"doc": method.__doc__}
      if 'usage' in method.__dict__:
        res["usage"] = method.usage
      return res
    
    tab_cmd = tab_cmd
    n = len(tab_cmd)
    
    # Handle empty command
    if(tab_cmd == ['']):
      return
    
    i = 0
    tab = dict([(token.text, token) for token in self.commands])
    
    end_command = None
    
    # For each token of the typed command
    while(i < n):
      word = tab_cmd[i]
      
      # If this token is in tab, it is authorized
      if(tab.has_key(word)):
      
        # If this token is final and can be followed by anything else, return now
        if tab[word].isUnilimited():
          return _getHelp(tab[word].final)
      
        # It is the last token ?
        if(i == n-1):
          
          # If the corresponding token is not final, error
          if(not tab[word].isFinal()):
            raise InvalidCommand()
          
          else: # Get the end callback help
            return _getHelp(tab[word].final)
            
        # Other tokens are coming => continue
        else:
          tab = tab[word].getNextTokens()
          
      else:
        raise InvalidCommand()
      
      i += 1
    
     
    
  def find_matches(self, text, dico):
    """Find matches for auto-completion"""
    matches = [value for value in dico
               if value.upper().find(text.upper(), 0, len(text)) == 0]
    return matches
    