#!/usr/bin/python
# -*- coding: iso-8859-15 -*-

from lib.exceptions import *
from itertools import tee
  
  
class Token:

  def __init__(self, text, tokens = [], final=None, unilimited=False):
    """Constructor"""
    self.text = text
    self.final = final
    self.tokens = tokens
    self.unilimited = unilimited
    
  def getNextTokens(self):
    """Get token objects that follow this one"""
    res = self.tokens
    if self.tokens.__class__ == TokenList:
      res = self.tokens.getTokens()
    return dict([(t.text, t) for t in res])
    
  def isFinal(self):
    """Return true if this token has a callback, false otherwise"""
    return (self.final is not None)
    
  def isUnilimited(self):
    """Can this token be followed by anything ?"""
    return self.unilimited
    
  def endCommand(self, args):
    """Callback to call when the command is parsed"""
    if self.final is not None:
      return self.final(args)
    else:
      raise Exception("No final callback defined!")
     
  def getPossibleCommands(self):
    """Return the list of all possible commands starting with this token, without inspecting TokenLists"""
    
    res = []
    if self.isFinal():
      res = [self.text]
    
    if self.tokens.__class__ != TokenList:
      for t in self.tokens:
        res += [self.text + " " + cmd for cmd in t.getPossibleCommands()]
    
    return res
      
    
"""
TokenList
"""
class TokenList:

  def __init__(self, callback, args=None, tokens = [], final=None):
    self.callback = callback
    self.args = args
    self.tokens = tokens
    self.final = final
  
  def getTokens(self):
    return [Token(res, self.tokens, self.final) for res in self.callback(self.args)]
    
    
  