<?php

/**
 * @class ApiException
 * @brief Main class for API exceptions
 */
class ApiException extends XeekException {

  /**
   * @brief Constructor
   * @param $error_num Error number
   * @param $error_msg Error message
   */
  public function __construct($error_num, $error_msg) {
    $this->error_num = $error_num;
    $this->error_msg = $error_msg;
  }
  
  /**
   * @brief Get an external representation of the exception
   * @return array representing the exception
   */
  public function serialize(){
    return array("error_num" => $this->error_num, 
                 "error_msg" => $this->error_msg);
  }
}

/**
 * @class InvalidMethod
 * @brief Raised when an invalid method is called
 */
class InvalidMethod extends ApiException {
  public function __construct() {
    parent::__construct(1, "Invalid method.");
  }
}

/**
 * @class MissingParameter
 * @brief Raised when a method is called with a missing parameter
 */
class MissingParameter extends ApiException {
  public function __construct($param) {
    parent::__construct(2, "Missing parameter: " . $param);
  }
}

/**
 * @class InvalidOutput
 * @brief Raised when we try to use an invalid API output format
 */
class InvalidOutput extends ApiException {
  public function __construct() {
    parent::__construct(3, "Invalid output.");
  }
}

/**
 * @class MissingMethod
 * @brief Raised when the method name is not provided when using the API
 */
class MissingMethod extends ApiException {
  public function __construct() {
    parent::__construct(4, "No method name.");
  }
}

/**
 * @class InvalidHTTPMethod
 * @brief Raised when an API method is called with an incorrect HTTP method
 */
class InvalidHTTPMethod extends ApiException {
  public function __construct() {
    parent::__construct(5, "Invalid HTTP method.");
  }
}

/**
 * @class InvalidParameter
 * @brief Raised when an API method is called with an invalid parameter
 */
class InvalidParameter extends ApiException {
  public function __construct($param) {
    parent::__construct(100, "Invalid parameter: " . $param);
  }
}

/**
 * @class InvalidParameterCombination
 * @brief Raised when an API method is called with an invalid combination of parameters
 */
class InvalidParameterCombination extends ApiException {
  public function __construct() {
    parent::__construct(101, "Invalid parameter combination.");
  }
}

/**
 * @class NotYetImplemented
 * @brief Raised when an API method is called but not yet implemented
 */
class NotYetImplemented extends ApiException {
  public function __construct() {
    parent::__construct(500, "Not yet implemented!");
  }
}

?>