<?php

/**
 * @class Config
 * @brief Server configuration management and statistics
 */
class Config {

  //Singleton instance
  protected static $s_instance = null;
  
  //ConfigEntry objects
  protected $entries;
  
  //Values
  protected $values;
  

  /**
   * @brief Constructor
   *        Don't call it directly! Use static method getInstance() instead.
   */
  protected function __construct(){
    $this->entries = ConfigEntry::fetchList();
    
    //Values
    $this->values = array();
    foreach($this->entries as $e){
      $this->values[$e->name] = $e->value;
    }
  }
  
  
  /**
   * @brief Get the unique instance of the Config class
   * @return unique instance
   */
  public static function getInstance() {
    if(!static::$s_instance){
      static::$s_instance = new Config();
    }
    return static::$s_instance;
  }
  
  /**
   * @brief Getter
   * @param $name property name
   */
  public function __get($name) {
    if(isset($this->values[$name])){
      return $this->values[$name];
    } else {
      throw new InternalException("Config::__get: Invalid property.");
    }
  }
  
  
  
  /**
   * @brief Serialize the object
   * @return external representation of the object
   */
  public function serialize(){
    $ser = array();
    
    //Add config entries
    foreach($this->entries as $e){
      $ser[$e->name] = $e->value;
    }
    
    return $ser;
  }

}

?>