<?php

/**
 * @class Format
 * @brief Decode strings into object, and encode objects into strings. Implement Proxy design pattern.
 */
abstract class Format {

  //Encoding / Decoding engine
  protected $engine;
  
  //Proxy functions
  protected $proxy_decode;
  protected $proxy_encode;
  
  /**
   * @brief Constructor
   * @param $engine encoding and decoding engine
   * @param $decode_func function that will be called when calling decode()
   * @param $encode_func function that will be called when calling encode()
   */
  public function __construct($engine, $decode_func = 'decode', $encode_func = 'encode'){
    $this->engine = $engine;
    $this->proxy_decode = $decode_func;
    $this->proxy_encode = $encode_func;
  }

  /**
   * @brief Decode a string
   * @result object
   */
  public function decode($request){
    return $this->engine->{$this->proxy_decode}($request);  //TODO use call_user_func instead
  }
  
  /**
   * @brief Encode an object
   * @result string
   */
  public function encode($result){
    return $this->engine->{$this->proxy_encode}($result);
  }
  
  /**
   * @brief Send appropriate headers
   */
  public abstract function send_header();
  
  
}

?>