<?php

/**
 * @class RPC
 * @brief RPC Handler base class
 */
abstract class RPC {

  //RPC Parameters
  protected $params;

  //Current session
  protected $session;
  
  //Core
  protected $core;
  
  
  /**
   * @brief Constructor
   * @param $session Session object to use. Try to detect it by default.
   */
  public function __construct($session = null) {
    global $_GET, $_POST;
    
    //Fetch parameters
    $this->params = $this->fetchParams($_GET, $_POST);
    
    //Detect the session
    $this->session = $session != null ? $session : $this->getSession();
    if($this->session == null) {
      die(defined("DEBUG") ? "Invalid session." : "");
    }
    
    //Get the core
    $this->core = Core::getInstance();
  }
  
  
  /**
   * @brief Get the RPC parameters. By default, use the 'params' POST parameter with JSON encoding.
   * @param $get_array GET array
   * @param $post_array POST array
   * @return parameter object (instance of stdClass)
   */
  protected function fetchParams($get_array, $post_array) {
    
    //Get the params
    if(isset($post_array["params"])) {
      $paramsJSON = $post_array["params"];
    } else {
      $paramsJSON = "{}";
    }
    
    //JSON-decode
    return Core::getInstance()->json->decode($paramsJSON);
  }
  
  
  /**
   * @brief Get the session. By default, we assume that we already have a session.
   * @return Session object, or null if no one can be created
   */
  protected function getSession() {
    global $_SERVER, $_COOKIE, $_GET;
    return SessionManager::getSession($_SERVER, $_COOKIE, $_GET);
  }
  
  

  /**
   * @brief Dispatch the request
   */
  public static function dispatchRequest() {
    global $_GET, $_POST;
    
    //Get the RPC name
    if(!isset($_GET["rpc"])) {
      die(defined("DEBUG") ? "Error: No RPC name." : "");
    }
    $rpc_name = $_GET["rpc"];
    
  
    //Instanciate the RPC service
    $rpc = new $rpc_name();
    
    //Call the handling method
    $rpc->handleRequest();
  }
  
  /**
   * @brief Handle the request
   */
  public abstract function handleRequest();

}

?>