<?php

require_once(dirname(__FILE__)."/../conf/mysql.php");

/**
 * @class MySQL
 * @brief MySQL wrapper for DB operations
 */
class MySQL {

  /**
   * @brief Connection handle
   */
  private static $db = null;

  /**
   * @brief Connect to the database if necessary
   */
  protected static function connect(){
    if(self::$db == null){
      self::$db = mysql_connect(MYSQL_HOST, MYSQL_USERNAME, MYSQL_PASSWORD);
      mysql_select_db(MYSQL_DATABASE, self::$db);
    }
  }

  /**
   * @brief Close the connection
   */
  protected static function close(){
    if(self::$db != null){
      mysql_close(self::$db);
      self::$db = null;
    }
  }
  
  /**
   * @brief Escape a string
   * @param $string string to escape
   * @return escaped string
   */
  public static function escape_string($string) {
    self::connect();
    return mysql_real_escape_string($string);
  }
  
  
  
  /**
   * @brief Execute a query and return the result as an associative array
   */
  public function query($query){
    return mysql_query($query);
  }
  
  
  /**
   * @brief Perform a select and fetch all results
   * @param $tables can be an array containing various table names, or one string for only one table
   * @param $fields 
   * @param $where_clause
   * @param $order_by
   * @param $limit
   * @param $distinct
   * @result list of arrays
   */
  public static function select($tables, $fields = "*", $where_clause = 1, $order_by = "id", $limit = "", $distinct = false){
    self::connect();
    
    if(is_array($tables)){
      $base_table = $tables[0];
      $tables = implode(',', $tables);
    } else {
      $base_table = $tables;
    }
    
    //Build the query
    $query = "SELECT ";
    if($distinct) $query .= "DISTINCT ";
    
    $n = sizeof($fields);
    $i = 0;
    foreach($fields as $f){
    
      if(preg_match("#\.#", $f)) {
        $field_name = "`" . preg_replace("#\.#", "`.`", self::escape_string($f)) . "`";
      } else {
        $field_name = "`" . $base_table . "`.`" . self::escape_string($f) . "`";
      }
      
      $query .= $field_name . (++$i != $n ? ", " : "");
    }
    
    $query .= " FROM " . self::escape_string($tables) . " WHERE " . $where_clause . " ORDER BY " . $order_by;
    
    if($limit != ""){
      $query .= " LIMIT ".$limit;
    }
    
    //Execute the query
    $res = self::query($query);// or die($query.": ".mysql_error());
    
    //Fetches the results in an associative array
    $tab = self::fetchAllAssoc($res);
    
    return $tab;
  }
  
  
  /**
   * @brief Perform a select and fetch all results
   * @param $table
   * @param $fields
   * @param $where_clause
   * @param $order_by
   * @result array or null if no result
   */
  public static function selectOne($table, $fields = "*", $where_clause = 1, $order_by = "id"){
    $res = self::select($table, $fields, $where_clause, $order_by, "1");
    
    if(count($res) == 0) {
      return null;
    }
    
    return $res[0];
  }
  
  
  
  /**
   * Fetch all results from a query
   * @param $res_query query result
   * @return array
   */
  public static function fetchAllAssoc($res_query){
    self::connect();
    $tab = array();
    
    $i = 0;
    while($t = mysql_fetch_assoc($res_query)){
      $tab[$i++] = $t;
    }
    
    return $tab;
  }
  
  
  /** 
   * @brief Insert a record into a table
   * @param $table name of the table
   * @param $values associative array field_name => field_value
   * @return the new id
   */
  public static function insert($table, $values){
    self::connect();
    
    //Build the query
    $query = "INSERT INTO `".self::escape_string($table)."` SET ";
    $n = sizeof($values);
    $i = 0;
    foreach($values as $name => $val){
      $query .= "`".self::escape_string($name)."` = '".self::escape_string($val)."'". (++$i != $n ? ", " : "");
    }
    
    //echo $query."\n";
    
    //Execute it
    $res = self::query($query);
    
    //Return the inserted ID
    return mysql_insert_id();
  }
  
  
  /** 
   * @brief Update a record
   * @param $table name of the table
   * @param $id id if the record to update
   * @param $values associative array field_name => field_value
   * @return true of false
   */
  public static function update($table, $id, $values){
    self::connect();
    
    //Build the query
    $query = "UPDATE `".self::escape_string($table)."` SET ";
    $n = sizeof($values);
    $i = 0;
    foreach($values as $name => $val){
      $query .= "`".self::escape_string($name)."` = '".self::escape_string($val)."'". (++$i != $n ? ", " : "");
    }
    
    $query .= " WHERE id = '".self::escape_string($id)."'";
    
    //Execute it
    return self::query($query);
  }
  

  
  /**
   * @brief Delete a or mor rows
   * @param $table table
   * @param $where_clause WHERE condition
   * @return true or false
   */
  public static function delete($table, $where_clause) {
    self::connect();
    
    //Build the query
    $query = "DELETE FROM `".self::escape_string($table)."` WHERE ".$where_clause;
    
    //Execute it
    return self::query($query);
  }
}

?>