#!/usr/bin/python
# -*- coding: iso-8859-15 -*-

from lib.api import *

from parser import *
from exceptions import *
from objects import *

MODE_EXT = 1
MODE_CREATE_CHAIN = 2
MODE_SELECT_CHAIN = 3
MODE_SELECT_AUDIT = 4
MODE_ADD_INST = 5
MODE_ADD_CHAIN = 6

class State:


  def __init__(self, api):
    '''
    Constructor
    '''
    
    # API
    self.api = api
  
    # Current and previous modes
    self.current_mode = MODE_EXT
    self.previous_modes = []
    
    # Current selected objects
    self.selected_audit = None
    
    # Current object to add
    self.instruction_to_add = None
    self.chain_to_add = None
    
    # Current objects
    self.sessions = []

    # Retrieve server objects
    self.config = self.getConfig()
    self.audit_list = self.getAuditList()
    # self.chain_list = self.getChainList()
    self.exploit_list = self.getExploitList()
    self.session_list = self.api.listSessions()
    self.result_list = self.api.listResults()
    
  
    # Prompt
    self.prompt = 'xeek > '
  
  
  def getAuditList(self):
    self.audit_list = self.api.listAudits()
    return self.audit_list
  
  def getChainList(self):
    self.chain_list = self.api.listChains()
    return self.chain_list
    
  def getExploitList(self):
    self.exploit_list = self.api.listExploits()
    return self.exploit_list
    
  def getSessionList(self):
    self.session_list = self.api.listSessions()
    return self.session_list
    
  def getResultList(self):
    self.result_list = self.api.listResults()
    return self.result_list
    
  def getQueue(self):
    if self.selected_audit is None:
      self.queue = None
    else:
      self.queue = self.api.getQueue(self.selected_audit.id)
    return self.queue
    
  def getConfig(self):
    self.config = self.api.getConfig()
    return self.config
    
  def getAudit(self, name):
    for obj in self.getAuditList():
      if obj.name == name:
        return obj
    return None
    
  def getChain(self, name):
    for obj in self.getChainList():
      if obj.name == name:
        return obj
    return None
    
    
  def getSelectedAuditId(self):
    if self.selected_audit is None:
      id = 0
    else:
      id = self.selected_audit.id
    return id
  
  
  def getSessionListFromSelectedAudit(self, online):
    self.sessions = self.api.listSessions(self.getSelectedAuditId(), online)
    return self.sessions
    
  def getResultListFromSelectedAudit(self):
    self.results = self.api.listResults(None, None, self.getSelectedAuditId())
    return self.results
  
  
  def switchMode(self, new_mode):
    self.previous_modes.append(self.current_mode)
    self.current_mode = new_mode
    self.updatePrompt()
    
  def backToPreviousMode(self):
    if(len(self.previous_modes) > 0):
      self.current_mode = self.previous_modes.pop()
      self.updatePrompt()
      
    
  def updatePrompt(self):
  
    if self.current_mode == MODE_EXT:
      self.prompt = 'xeek'
      
    else:
    
      self.prompt = 'xeek audit('+self.selected_audit.name+')'
      
      if self.current_mode == MODE_ADD_INST:
        self.prompt += '/inst('+self.instruction_to_add.exploit.id+')'
      
    self.prompt += ' > '
      
    #elif self.current_mode == MODE_SELECT_CHAIN:
    #  self.prompt = 'xeek chain('+self.selected_chain.name+') > '
    #elif self.current_mode == MODE_CREATE_CHAIN:
    #  self.prompt = 'xeek chain/create > '
    #elif self.current_mode == MODE_ADD_CHAIN:
    #  self.prompt += '/chain('+self.chain_to_add.exploit.id+') > '
      
  def pushCurrentInstructionToQueue(self):
    self.api.pushInstructionToQueue(self.selected_audit.id, 
                                    self.instruction_to_add.exploit.id,
                                    self.instruction_to_add.params, 
                                    self.instruction_to_add.session_id, 
                                    self.instruction_to_add.prio)
  
  def getURLAudit(self, audit):
    return self.getConfig()["rpc_url"]+"?rpc=InitSessionRPC&token="+audit.token
    