<?php

require_once(dirname(__FILE__)."/ApiException.class.php");

/**
 * @class HttpApi
 * @brief Let users call the API using HTTP protocol.
 *        Receive requests, forward them to the core, get the results, encode them and send them back to the client.
 */
class HttpApi {

  //Supported outputs formats array
  protected $out_formats;
  
  //Singleton instance
  protected static $s_instance = null;
  
  //API method and the corresponding HTTP method to call them
  protected static $s_http_methods = array( "audit/add"                => "POST",
                                            "audit/get"                => "GET",
                                            "audit/list"               => "GET",
                                            "audit/delete"             => "POST",
                                            
                                            "queue/get"                => "GET",
                                            "queue/instruction/add"    => "POST",
                                            //"queue/chain/add"          => "POST",
                                            "queue/instruction/delete" => "POST",
                                            "queue/clean"              => "POST",
                                            
                                            "session/list"             => "GET",
                                            "session/get"              => "GET",
                                            "session/reset"            => "POST",
                                            "session/delete"           => "POST",
                                            "session/clean"            => "POST",
                                            
                                            //"chain/list"               => "GET",
                                            //"chain/get"                => "GET",
                                            //"chain/add"                => "POST",
                                            //"chain/delete"             => "POST",
                                            
                                            "result/list"              => "GET",
                                            "result/get"               => "GET",
                                            "result/delete"            => "POST",
                                            
                                            "exploit/list"             => "GET",
                                            "exploit/get"              => "GET",
                                            
                                            "config/get"               => "GET",
                                            );
  

  /**
   * @brief Constructor
   */
  private function __construct() {
    $j = new JsonFormat();
    $this->out_formats = array( "default" => $j,
                                "json"    => $j); //TODO Add XML output
    $this->api = CoreApi::getInstance();
  }
  
  /**
   * @brief Get the unique instance of the Core class
   * @return unique instance
   */
  public static function getInstance() {
    if(!self::$s_instance){
      self::$s_instance = new HttpApi();
    }
    return self::$s_instance;
  }
  
  /**
   * @brief Dispatch a request
   */
  public function dispatchRequest(){
    global $_GET, $_POST;
    $req_params = array("GET" => $_GET, "POST" => $_POST);
    
    try {
    
      //Get the method name
      if(!isset($_GET["method"])) {
        throw new MissingMethod();
      }
      $method_name = $_GET["method"];
      
      //Get the corresponding HTTP method
      if(!array_key_exists($method_name, static::$s_http_methods)) {
        echo $method_name;
        throw new InvalidMethod();
      }
      $http_method = static::$s_http_methods[$method_name];
      
      //Check if the appropriate HTTP method was used
      if($_SERVER['REQUEST_METHOD'] != $http_method){
        throw new InvalidHTTPMethod();
      }
      
      //Get the params using the good HTTP method
      $params = $req_params[$http_method];
      
      //Get the output engine
      if(!isset($_GET["output"])) {
        $output = "default";
      } else {
        $output = $_GET["output"];
      }
      if(!in_array($output, array_keys($this->out_formats))){
        throw new InvalidOutput();
      }
      $output_engine = $this->out_formats[$output];
      
      //Call the API method, get the result
      $result = $this->api->callMethod($method_name, $params);
      
      //Apply the output transformation
      $result_encoded = $output_engine->encode($result);
      
      //Output the result
      $output_engine->send_header();
      echo $result_encoded;
      
    } catch(ApiException $e) {
      $this->out_formats["default"]->send_header();
      echo $this->out_formats["default"]->encode($e->serialize());
    }
  }
  
}

?>