<?php

/**
 * @class Audit
 * @brief Represent a targeted attack on various victims
 */
class Audit extends DBObject {

  //Name
  public $name;
  
  //Exploit queue
  public $queue;
  
  //Created at
  public $created_at;
  
  
  /**
   * @brief Constructor
   * @param $params Associative array: property => value
   * @param $isFetch True if the object has been fetched from the database
   */
  public function __construct($params, $isFetch = false) {
  
    //Generate a unique token if necessary
    if(!$isFetch) {
      do {
        $token = static::genToken();
      } while (static::fetchByToken($token) != null);
      $params["token"] = $token;
    }
  
    //Create the DB object
    parent::__construct($params, $isFetch);
    
    //fetch the queue
    $this->queue = new Queue($this);
    
    //Enable serialization
    $this->setSerializable(array('id', 'name', 'token', 'created_at'));
  }

  
  /**
   * @brief For display
   */
  public function __toString(){
    return $this->name;
  }
  
  
  /**
   * @brief Standard DBObject methods
   */
  public static function getClass(){
    return "Audit";
  }
  
  public static function getTable(){
    return "audit";
  }
  
  public static function getAutoProps(){
    return array("name", "token", "created_at");
  }
  

}

?>