<?php

/**
 * @class Chain
 * @brief Represent a chain of exploits
 */
class Chain extends DBObject {

  //Name of the chain
  protected $name;
  
  //Description
  protected $description;

  //Exploit list
  protected $exploits;
  
  /**
   * @brief Constructor.
   * @param $args Associative array: property => value
   * @param $isFetch True if the object has been fetched from the database
   */
  public function __construct($args = null, $isFetch = false) {
  
    //Call the parent constructor
    parent::__construct($args, $isFetch);
    
    if($isFetch){
      $this->exploits = ChainedExploit::fetchList("chain_id = '".intval($this->id)."'", "id");
    } else {
      $this->exploits = array();
    }
    
    //Enable serialization
    $this->setSerializable(array('id', 'name', 'description'));
  }

  /**
   * @brief Add an exploit at the end of the chain
   * @param $exploit_name Name of the exploit
   */
  public function addExploit($exploit_name) {
  
    if($this->id == 0) {
      throw new MustSaveChainBeforeAdd();
    }
  
    //TODO check sur $exploit_name
    $e = new ChainedExploit(array("chain_id" => $this->id,
                                  "exploit_name" => $exploit_name));
    $e->save();
    $this->exploits[] = $e;
  }
  
  
  /**
   * @brief Get the list of exploits
   * @return array of exploit names
   */
  public function getExploits() {
    $res = array();
    
    foreach($this->exploits as $e){
      $res[] = $e->exploit_name;
    }
    
    return $res;
  }
  
  
  /**
   * @brief Serialize the object
   * @return array representing the object
   */
  public function serialize() {
    $a = parent::serialize();
    $a["exploits"] = array();
    foreach($this->exploits as $i){
      $a["exploits"][] = $i->serialize();
    }
    return $a;
  }
    
  
  public static function getClass(){
    return "Chain";
  }
  
  public static function getTable(){
    return "chain";
  }
  
  public static function getAutoProps(){
    return array("name", "description");
  }

}

?>