<?php

/**
 * @class XeekException
 * @brief The main exception class
 */
class XeekException extends Exception { }

/**
 * @class InternalException
 * @brief For all Xeek internal exceptions
 */
class InternalException extends XeekException { }

/**
 * @class UnknownClass
 * @brief Raised when an invalid class is used
 */
class UnknownClass extends InternalException {
  public function __construct($class){
    parent::__construct("Unable to find class '$class'.");
  }
}

/**
 * @class InvalidSession
 * @brief Raised when an invalid class is used
 */
class InvalidSession extends InternalException {
  public function __construct(){
    parent::__construct("The provided session id is incorrect.");
  }
}

/**
 * @class InvalidAudit
 * @brief Raised when an invalid class is used
 */
class InvalidAudit extends InternalException {
  public function __construct(){
    parent::__construct("The audit id is incorrect.");
  }
}

/**
 * @class InvalidChain
 * @brief Raised when we try to use an invalid chain
 */
class InvalidChain extends InternalException {
  public function __construct(){
    parent::__construct("The provided chain id is incorrect.");
  }
}

/**
 * @class MustSaveChainBeforeAdd
 * @brief Raised when we try to add an exploit to a chain without saving it first
 */
class MustSaveChainBeforeAdd extends InternalException {
  public function __construct(){
    parent::__construct("You cannot add an exploit to the chain before saving it.");
  }
}

/**
 * @class InvalidExploit
 * @brief Raised when we try to use an invalid exploit
 */
class InvalidExploit extends InternalException {
  public function __construct(){
    parent::__construct("The provided exploit id is incorrect.");
  }
}

/**
 * @class MissingExploitParameter
 * @brief Raised when a parameter is missing
 */
class MissingExploitParameter extends InternalException {
  public function __construct($param){
    parent::__construct("The '$param' parameter is missing.");
  }
}

/**
 * @class IncorrectNumberOfParameters
 * @brief Raised when the number of exploit parameters is incorrect
 */
class IncorrectNumberOfParameters extends InternalException {
  public function __construct(){
    parent::__construct("The number of provided parameter lists does not correspond to the number of exploits in the chain.");
  }
}

?>