<?php

/**
 * @class InitSessionRPC
 * @brief Session initialization RPC Handler
 */
class InitSessionRPC extends RPC {

  /**
   * @brief Get the session. Since this is a specific service which is called at the very start of the audit, we are not yet authenticated.
   * @return Session object, or null if no one can be created
   */
  protected function getSession() {
    global $_SERVER, $_COOKIE, $_GET;
    
    //Check if the token parameter is defined
    if(!property_exists($this->params, 'token')) {
      die(defined("DEBUG") ? "Token parameter not found." : "");
    }
    
    //Detect or create the session
    return SessionManager::initSession($this->params->token, $_SERVER, $_COOKIE, $_GET);
  }
  
  /**
   * @brief Get the RPC parameters. This specific service uses the GET 'token' parameter.
   * @param $get_array GET array
   * @param $post_array POST array
   * @return parameter object
   */
  protected function fetchParams($get_array, $post_array) {
    $params = new stdClass();
  
    if(isset($get_array["token"])) {
      $params->token = $get_array["token"];
    }
  
    return $params;
  }


  /**
   * @brief Handle the request
   */
  public function handleRequest() {
  
    //Set the content-type
    @header("Content-type: application/x-javascript");
  
    //Display XeegAgent code
    echo $this->core->getXeekAgentCode(SessionManager::SESSID_PARAM_NAME, $this->session->xeek_cookie);
    
    //Display code of all registered exploits
    foreach(Exploit::getExploitList() as $exploit){
      echo $exploit->getJSLibCode();
    }
    
    //Fetch instructions, if any, and display them inside the Xeek_first_exploits() function
    echo "function Xeek_first_exploits() {\n";
    $rpc = new InstructionFetchRPC($this->session);
    $rpc->handleRequest();
    echo "}";
  }

}

?>