<?php

/**
 * @class Instruction
 * @brief Exploit instruction that populates the queue.
 */
class Instruction extends DBObject {

  //Audit ID
  protected $audit_id;
  
  //Session ID (0 if no session)
  protected $session_id;
  
  //Token
  protected $token;
  
  //Name of the exploit
  public $exploit_id;
  
  //Parameters of the instruction, encoded in JSON format
  protected $params;
  
  //Priority of the instruction
  protected $prio;
  
  
  //Associated exploit object
  protected $exploit;
  
  //Priority levels
  const PRIO_DEFAULT = 0;
  const PRIO_MAX = 10;
  
  
  /**
   * @brief Constructor. You're not supposed to call this directly from the outside; use Queue::pushInstr() instead.
   * @param $args Associative array: property => value
   * @param $isFetch True if the object has been fetched from the database
   */
  public function __construct($args, $isFetch = false) {
  
    //Generate a unique token if necessary
    if(!$isFetch) {
      do {
        $token = static::genToken();
      } while (static::fetchByToken($token) != null);
      $args["token"] = $token;
    }
    
    //session_id is 0 by default
    if(!isset($args["session_id"])){
      $args["session_id"] = 0;
    } 
    
    //If it is set, check the session
    if($args["session_id"] != 0) { 
      if(Session::fetchById($args["session_id"]) == null){
        throw new InvalidSession();
      }
    }
  
    //Call the parent constructor
    parent::__construct($args, $isFetch);
    
    //Get the associated exploit
    $this->exploit = Exploit::getExploit($this->exploit_id);
    
    //Check the exploit
    if($this->exploit == null){
      throw new InvalidExploit();
    }
    
    //Check the params parameters
    $this->exploit->checkParams($this->params);
    
    //Enable serialization
    $this->setSerializable(array('id', 'token', 'audit_id', 'session_id', 'exploit_id', 'params', 'prio', 'created_at'));
  }
  
  /** 
   * @brief Save the object
   * @param $props Object properties that will be saved into the database
   */
  protected function _save($props){
  
    //Convert the prams array into a JSON one.
    $props["params"] = Core::getInstance()->json->encode($props["params"]);;
  
    //Save the properties
    parent::_save($props);
  }
  
  
  /**
   * @brief Post fetch instruction (decoding)
   */
  protected function postFetch(){
    //Decode the params property
    $this->params = Core::getInstance()->json->decode($this->params);
  }
  
  
  /**
   * @brief Get the JS code of the exploit
   * @return JS code
   */
  public function getJSCode(){
    if($this->exploit == null) throw new InvalidExploit();
    return $this->exploit->getJSCode($this->token, $this->params);
  }

  public static function getClass(){
    return "Instruction";
  }
  
  public static function getTable(){
    return "instruction";
  }
  
  public static function getAutoProps(){
    return array("audit_id", "session_id", "token", "exploit_id", "params", "prio", "created_at");
  }

}

?>