<?php

/**
 * @class Module
 * @brief Represent a XeeK module
 */
abstract class Module {

  const TYPE_EXPLOIT = 1;
  
  //Module type
  protected $type;

  //Name of the module
  protected $name = "";
  
  //Description
  protected $description = "";
  
  //Authors
  protected $authors = array();
  
  //Version
  protected $version = "1.0";
  
  //Websites
  protected $websites = array();
  
  //Propertiy names
  protected static $prop_names = array("name", "description", "authors", "version", "websites");
  
  /** 
   * @brief Protected contructor
   * @param $header_file header file path
   */
  protected function __construct($header_file){
  
    //Parse the header
    $this->parseHeader($this->readHeader($header_file));
  }
  
  /**
   * @brief Parse the module header 
   * @param $header header string
   * @return property array
   */
  protected function parseHeader($header) {
    
    //Preliminary check
    if(preg_match("#/\*(.+?)\*/#s", $header, $res)){
      $header = $res[1];
      $tab = preg_split("#[\r\n]+#", $header);
      
      //Split the header into lines
      foreach($tab as $line){
      
        //Parse each line to get properties
        if(preg_match("#\s*\*\s*([a-z]+)\s*:\s*(.+)#i", $line, $res)){
          $name = strtolower($res[1]);
          
          //Deal with the "authors" property
          if($name == "author" || $name == "authors") {
            $this->authors = preg_split("#,\s*#", $res[2]);
          } elseif(in_array($name, static::$prop_names)){
            $this->$name = $res[2];
          }
        }
      }
    }
  }
  
  /**
   * @brief Read the header of a module file
   * @param $module_path module path
   * @return header of empty string if no header was found
   */
  protected function readHeader($module_path){
    $header = "";
    $path = dirname(__FILE__)."/../modules/".$module_path;
    
    if(preg_match("#/\*.+?\*/#s", file_get_contents($path), $res)){
      $header = $res[0];
    }
    
    return $header;
  }
  
  
  /**
   * @brief Seraialize the module
   * @return external representation of the module
   */
  public function serialize(){
    $res = array();
    foreach(static::$prop_names as $prop_name){
      $res[$prop_name] = $this->$prop_name;
    }
    return $res;
  }
  
  
}

?>