<?php

/**
 * @class Result
 * @brief Result of an exploit.
 */
class Result extends DBObject {

  //ID of the session
  protected $session_id;
  
  //ID of the instruction
  protected $instruction_id;
  
  //Result data, encoded in JSON format
  protected $data;
  
  //Creation date
  protected $created_at;
  
  //Exploit id
  protected $exploit_id;
  
  
  /**
   * @brief Constructor. You're not supposed to call this directly from the outside; use Queue::pushInstr() instead.
   * @param $args Associative array: property => value
   * @param $isFetch True if the object has been fetched from the database
   */
  public function __construct($args, $isFetch = false) {
  
    $this->exploit_id = "";
    
    //Call the parent constructor
    parent::__construct($args, $isFetch);
    
    //Enable serialization
    $this->setSerializable(array('id', 'session_id', 'instruction_id', 'exploit_id', 'data', 'created_at'));
  }
  
  /** 
   * @brief Save the object
   * @param $props Object properties that will be saved into the database
   */
  protected function _save($props){
  
    //Convert the data array into a JSON one.
    $props["data"] = Core::getInstance()->json->encode($props["data"]);
  
    //Save the properties
    parent::_save($props);
  }
   
  
  /**
   * @brief Post fetch instruction
   */
  protected function postFetch(){
    //Decode the data property
    $this->data = Core::getInstance()->json->decode($this->data);
    
    //Get the corresponding exploit id
    $i = Instruction::fetchById($this->instruction_id);
    if($i != null){
      $this->exploit_id = $i->exploit_id;
    }
  }
  
  /**
   * @brief List results by session ID or instruction ID.
   * @param $session_id Session ID. Ignored if null.
   * @param $instruction_id Instruction ID. Ignored if null.
   * @return array of result objects, the most recent first
   */
  public static function listBySessionAndInstruction($session_id = null, $instruction_id = null){
    $where_clause = "1";
    if($session_id){
      $where_clause .= " AND session_id = '".intval($session_id)."'";
    }
    if($instruction_id){
      $where_clause .= " AND instruction_id = '".intval($instruction_id)."'";
    }
    
    return static::fetchList($where_clause, "created_at");
  }
  
  /**
   * @brief List results by audit ID
   * @param $audit_id audit ID
   * @return array of result objects, the most recent first
   */
  public static function listByAudit($audit_id){
    return static::fetchWithJoin( array("session", "instruction"),
                                  "session.audit_id = '".intval($audit_id)."' AND result.instruction_id = instruction.id", 
                                  "created_at");
  }
  
  
  public static function getClass(){
    return "Result";
  }
  
  public static function getTable(){
    return "result";
  }
  
  public static function getAutoProps(){
    return array('session_id', 'instruction_id', 'data', 'created_at');
  }

}

?>